@@ -9,7 +9,7 @@ import sys
 import time
 
 __copyright__ = """
-Copyright (C) 2007-2017 David Aguilar and contributors
+Copyright (C) 2007-2022 David Aguilar and contributors
 """
 
 try:
@@ -17,10 +17,12 @@ try:
 except ImportError:
     sys.stderr.write(
         """
-You do not seem to have PyQt5, PySide, or PyQt4 installed.
-Please install it before using git-cola, e.g. on a Debian/Ubutnu system:
+You do not seem to have PyQt (or PySide) and qtpy installed.
+Please install the PyQt (or PySide) and qtpy into your Python environment.
 
-    sudo apt-get install python-pyqt5 python-pyqt5.qtwebkit
+For example, on a Debian/Ubuntu system you can install these using apt:
+
+    sudo apt install python3-pyqt5 python3-pyqt5.qtwebkit python3-qtpy
 
 """
     )
@@ -193,6 +195,7 @@ class ColaApplication(object):
         self._install_hidpi_config()
         self._app = ColaQApplication(context, list(argv))
         self._app.setWindowIcon(icons.cola())
+        self._app.setDesktopFileName('git-cola')
         self._install_style(gui_theme)
 
     def _install_style(self, theme_str):
@@ -578,6 +581,20 @@ class Timer(object):
         sys.stdout.write('%s: %.5fs\n' % (key, elapsed))
 
 
+class NullArgs(object):
+    """Stub arguments for interactive API use"""
+
+    def __init__(self):
+        self.icon_themes = []
+        self.theme = None
+        self.settings = None
+
+
+def null_args():
+    """Create a new instance of application arguments"""
+    return NullArgs()
+
+
 class ApplicationContext(object):
     """Context for performing operations on Git and related data models"""
 
@@ -617,9 +634,7 @@ def find_git():
     # If the user wants to use a Git/bin/ directory from a non-standard
     # directory then they can write its location into
     # ~/.config/git-cola/git-bindir
-    git_bindir = os.path.expanduser(
-        os.path.join('~', '.config', 'git-cola', 'git-bindir')
-    )
+    git_bindir = resources.config_home('git-bindir')
     if core.exists(git_bindir):
         custom_path = core.read(git_bindir).strip()
         if custom_path and core.exists(custom_path):
