@@ -62,7 +62,7 @@ class EditModel(ContextCommand):
 
     def do(self):
         """Perform the operation."""
-        self.model.set_filename(self.new_filename)
+        self.model.filename = self.new_filename
         self.model.set_mode(self.new_mode)
         self.model.set_diff_text(self.new_diff_text)
         self.model.set_diff_type(self.new_diff_type)
@@ -70,7 +70,7 @@ class EditModel(ContextCommand):
 
     def undo(self):
         """Undo the operation."""
-        self.model.set_filename(self.old_filename)
+        self.model.filename = self.old_filename
         self.model.set_mode(self.old_mode)
         self.model.set_diff_text(self.old_diff_text)
         self.model.set_diff_type(self.old_diff_type)
@@ -348,7 +348,7 @@ class ApplyPatches(ContextCommand):
 
 
 class Archive(ContextCommand):
-    """"Export archives using the "git archive" command"""
+    """ "Export archives using the "git archive" command"""
 
     def __init__(self, context, ref, fmt, prefix, filename):
         super(Archive, self).__init__(context)
@@ -1670,37 +1670,58 @@ class OpenDefaultApp(ContextCommand):
 
     def __init__(self, context, filenames):
         super(OpenDefaultApp, self).__init__(context)
-        if utils.is_darwin():
-            launcher = 'open'
-        else:
-            launcher = 'xdg-open'
-        self.launcher = launcher
         self.filenames = filenames
 
     def do(self):
         if not self.filenames:
             return
-        core.fork([self.launcher] + self.filenames)
+        utils.launch_default_app(self.filenames)
 
 
-class OpenParentDir(OpenDefaultApp):
+class OpenDir(OpenDefaultApp):
+    """Open directories using the OS default."""
+
+    @staticmethod
+    def name():
+        return N_('Open Directory')
+
+    @property
+    def _dirnames(self):
+        return self.filenames
+
+    def do(self):
+        dirnames = self._dirnames
+        if not dirnames:
+            return
+        # An empty dirname defaults to CWD.
+        dirs = [(dirname or core.getcwd()) for dirname in dirnames]
+        utils.launch_default_app(dirs)
+
+
+class OpenParentDir(OpenDir):
     """Open parent directories using the OS default."""
 
     @staticmethod
     def name():
         return N_('Open Parent Directory')
 
-    def __init__(self, context, filenames):
-        OpenDefaultApp.__init__(self, context, filenames)
-
-    def do(self):
-        if not self.filenames:
-            return
+    @property
+    def _dirnames(self):
         dirnames = list(set([os.path.dirname(x) for x in self.filenames]))
-        # os.path.dirname() can return an empty string so we fallback to
-        # the current directory
-        dirs = [(dirname or core.getcwd()) for dirname in dirnames]
-        core.fork([self.launcher] + dirs)
+        return dirnames
+
+
+class OpenWorktree(OpenDir):
+    """Open worktree directory using the OS default."""
+
+    @staticmethod
+    def name():
+        return N_('Open Worktree')
+
+    # The _unused parameter is needed by worktree_dir_action() -> common.cmd_action().
+    def __init__(self, context, _unused=None):
+        dirnames = [context.git.worktree()]
+        super(OpenWorktree, self).__init__(context, dirnames)
 
 
 class OpenNewRepo(ContextCommand):
@@ -1731,7 +1752,7 @@ class OpenRepo(EditModel):
         if self.model.set_worktree(self.repo_path):
             self.fsmonitor.stop()
             self.fsmonitor.start()
-            self.model.update_status()
+            self.model.update_status(reset=True)
             # Check if template should be loaded
             if self.context.cfg.get(prefs.AUTOTEMPLATE):
                 template_loader = LoadCommitMessageFromTemplate(self.context)
@@ -2342,7 +2363,7 @@ def is_conflict_free(path):
     """Return True if `path` contains no conflict markers"""
     rgx = re.compile(r'^(<<<<<<<|\|\|\|\|\|\|\||>>>>>>>) ')
     try:
-        with core.xopen(path, 'r') as f:
+        with core.xopen(path, 'rb') as f:
             for line in f:
                 line = core.decode(line, errors='ignore')
                 if rgx.match(line):
@@ -2512,6 +2533,7 @@ class StageModifiedAndUntracked(StageCarefully):
 
 class StageOrUnstageAll(ContextCommand):
     """If the selection is staged, unstage it, otherwise stage"""
+
     @staticmethod
     def name():
         return N_('Stage / Unstage All')
