@@ -7,8 +7,10 @@ from os.path import join
 import re
 import struct
 
+from qtpy import QtCore
+from qtpy.QtCore import Signal
+
 from . import core
-from . import observable
 from . import utils
 from . import version
 from .compat import int_types
@@ -32,19 +34,25 @@ def _stat_info(git):
     # Try /etc/gitconfig as a fallback for the system config
     paths = [
         ('system', '/etc/gitconfig'),
-        ('user', _USER_XDG_CONFIG),
         ('user', _USER_CONFIG),
+        ('user', _USER_XDG_CONFIG),
     ]
     config = git.git_path('config')
     if config:
         paths.append(('repo', config))
 
     statinfo = []
+    categories_completed = set()
     for category, path in paths:
+        # We should only read ~/.gitconfig xor ~/.config/git/config (not both).
+        # Only processing the first entry in each category to accomplish this.
+        if category in categories_completed:
+            continue
         try:
             statinfo.append((category, path, core.stat(path).st_mtime))
         except OSError:
             continue
+        categories_completed.add(category)
     return statinfo
 
 
@@ -52,8 +60,8 @@ def _cache_key(git):
     # Try /etc/gitconfig as a fallback for the system config
     paths = [
         '/etc/gitconfig',
-        _USER_XDG_CONFIG,
         _USER_CONFIG,
+        _USER_XDG_CONFIG,
     ]
     config = git.git_path('config')
     if config:
@@ -104,15 +112,16 @@ def _config_key_value(line, splitchar):
     return k, _config_to_python(v)
 
 
-class GitConfig(observable.Observable):
+class GitConfig(QtCore.QObject):
     """Encapsulate access to git-config values."""
 
-    message_user_config_changed = 'user_config_changed'
-    message_repo_config_changed = 'repo_config_changed'
-    message_updated = 'updated'
+    user_config_changed = Signal(str, object)
+    repo_config_changed = Signal(str, object)
+    updated = Signal()
 
     def __init__(self, context):
-        observable.Observable.__init__(self)
+        super(GitConfig, self).__init__()
+        self.context = context
         self.git = context.git
         self._map = {}
         self._system = {}
@@ -204,7 +213,7 @@ class GitConfig(observable.Observable):
         for dct in (self._system, self._user, self._repo):
             self._all.update(dct)
 
-        self.notify_observers(self.message_updated)
+        self.updated.emit()
 
     def read_config(self, path):
         """Return git config data from a path as a dictionary."""
@@ -213,11 +222,11 @@ class GitConfig(observable.Observable):
             return self._read_config_file(path)
 
         dest = {}
-        if version.check_git(self, 'config-includes'):
+        if version.check_git(self.context, 'config-includes'):
             args = ('--null', '--file', path, '--list', '--includes')
         else:
             args = ('--null', '--file', path, '--list')
-        config_lines = self.git.config(*args)[STDOUT].split('\0')
+        config_lines = self.git.config(_readonly=True, *args)[STDOUT].split('\0')
         for line in config_lines:
             if not line:
                 # the user has an invalid entry in their git config
@@ -311,11 +320,13 @@ class GitConfig(observable.Observable):
 
         get_all() allows for this use case by gathering all of the per-config
         values separately and then orders them according to the expected
-        local > global > system order.
+        local > user > system precedence order.
 
         """
         result = []
-        status, out, _ = self.git.config(key, z=True, get_all=True, show_origin=True)
+        status, out, _ = self.git.config(
+            key, z=True, get_all=True, show_origin=True, _readonly=True
+        )
         if status == 0:
             current_source = ''
             current_result = []
@@ -347,23 +358,22 @@ class GitConfig(observable.Observable):
 
     def set_user(self, key, value):
         if value in (None, ''):
-            self.git.config('--global', key, unset=True)
+            self.git.config('--global', key, unset=True, _readonly=True)
         else:
-            self.git.config('--global', key, python_to_git(value))
+            self.git.config('--global', key, python_to_git(value), _readonly=True)
         self.update()
-        msg = self.message_user_config_changed
-        self.notify_observers(msg, key, value)
+        self.user_config_changed.emit(key, value)
 
     def set_repo(self, key, value):
         if value in (None, ''):
-            self.git.config(key, unset=True)
+            self.git.config(key, unset=True, _readonly=True)
         else:
-            self.git.config(key, python_to_git(value))
+            self.git.config(key, python_to_git(value), _readonly=True)
         self.update()
-        msg = self.message_repo_config_changed
-        self.notify_observers(msg, key, value)
+        self.repo_config_changed.emit(key, value)
 
     def find(self, pat):
+        """Return a a dict of values for all keys matching the specified pattern"""
         pat = pat.lower()
         match = fnmatch.fnmatch
         result = {}
@@ -424,7 +434,7 @@ class GitConfig(observable.Observable):
     def check_attr(self, attr, path):
         """Check file attributes for a path"""
         value = None
-        status, out, _ = self.git.check_attr(attr, '--', path)
+        status, out, _ = self.git.check_attr(attr, '--', path, _readonly=True)
         if status == 0:
             header = '%s: %s: ' % (path, attr)
             if out.startswith(header):
@@ -461,12 +471,12 @@ class GitConfig(observable.Observable):
             if utils.is_win32():
                 # Try to find Git's sh.exe directory in
                 # one of the typical locations
-                pf = os.environ.get('ProgramFiles', 'C:\\Program Files')
-                pf32 = os.environ.get('ProgramFiles(x86)', 'C:\\Program Files (x86)')
-                pf64 = os.environ.get('ProgramW6432', 'C:\\Program Files')
+                pf = os.environ.get('ProgramFiles', r'C:\Program Files')
+                pf32 = os.environ.get('ProgramFiles(x86)', r'C:\Program Files (x86)')
+                pf64 = os.environ.get('ProgramW6432', r'C:\Program Files')
 
                 for p in [pf64, pf32, pf, 'C:\\']:
-                    candidate = os.path.join(p, 'Git\\bin\\sh.exe')
+                    candidate = os.path.join(p, r'Git\bin\sh.exe')
                     if os.path.isfile(candidate):
                         return candidate
                 return None
@@ -508,3 +518,11 @@ def python_to_git(value):
     if isinstance(value, int_types):
         return ustr(value)
     return value
+
+
+def get_remotes(cfg):
+    """Get all of the configured git remotes"""
+    # Gather all of the remote.*.url entries.
+    prefix = len('remote.')
+    suffix = len('.url')
+    return sorted(key[prefix:-suffix] for key in cfg.find('remote.*.url'))
