@@ -1,9 +1,11 @@
-from __future__ import division, absolute_import, unicode_literals
+from __future__ import absolute_import, division, print_function, unicode_literals
 import sys
 
+from qtpy import QtCore
+from qtpy.QtCore import Signal
+
 from .. import core
 from .. import hidpi
-from .. import observable
 from .. import utils
 from ..cmd import Command
 
@@ -32,6 +34,7 @@ MERGE_KEEPBACKUP = 'merge.keepbackup'
 MERGE_SUMMARY = 'merge.summary'
 MERGE_VERBOSITY = 'merge.verbosity'
 MERGETOOL = 'merge.tool'
+RESIZE_BROWSER_COLUMNS = 'cola.resizebrowsercolumns'
 SAFE_MODE = 'cola.safemode'
 SAVEWINDOWSETTINGS = 'cola.savewindowsettings'
 SHOW_PATH = 'cola.showpath'
@@ -70,6 +73,7 @@ class Defaults(object):
     merge_keep_backup = True
     merge_summary = True
     merge_verbosity = 2
+    resize_browser_columns = False
     save_window_settings = True
     safe_mode = False
     autocomplete_paths = True
@@ -114,7 +118,7 @@ def display_untracked(context):
 
 def editor(context):
     """Return the configured editor"""
-    app = context.cfg.get(EDITOR, default=Defaults.editor)
+    app = context.cfg.get(EDITOR, default=fallback_editor())
     return _remap_editor(app)
 
 
@@ -124,6 +128,27 @@ def background_editor(context):
     return _remap_editor(app)
 
 
+def fallback_editor():
+    """Return a fallback editor for cases where one is not configured
+
+    GIT_VISUAL and VISUAL are consulted before GIT_EDITOR and EDITOR to allow
+    configuring a visual editor for Git Cola using $GIT_VISUAL and an alternative
+    editor for the Git CLI.
+    """
+    editor_variables = (
+        'GIT_VISUAL',
+        'VISUAL',
+        'GIT_EDITOR',
+        'EDITOR',
+    )
+    for env in editor_variables:
+        env_editor = core.getenv(env)
+        if env_editor:
+            return editor
+
+    return Defaults.editor
+
+
 def _remap_editor(app):
     """Remap a configured editorinto a visual editor name"""
     # We do this for vim users because this configuration is convenient.
@@ -207,13 +232,13 @@ def status_show_totals(context):
     return context.cfg.get(STATUS_SHOW_TOTALS, default=Defaults.status_show_totals)
 
 
-class PreferencesModel(observable.Observable):
+class PreferencesModel(QtCore.QObject):
     """Interact with repo-local and user-global git config preferences"""
 
-    message_config_updated = 'config_updated'
+    config_updated = Signal(str, str, object)
 
     def __init__(self, context):
-        observable.Observable.__init__(self)
+        super(PreferencesModel, self).__init__()
         self.context = context
         self.config = context.cfg
 
@@ -223,8 +248,7 @@ class PreferencesModel(observable.Observable):
             self.config.set_repo(config, value)
         else:
             self.config.set_user(config, value)
-        message = self.message_config_updated
-        self.notify_observers(message, source, config, value)
+        self.config_updated.emit(source, config, value)
 
     def get_config(self, source, config):
         """Get a configured value"""
