@@ -442,9 +442,11 @@ def open_files(title, directory=None, filters=''):
 
 def opendir_dialog(caption, path):
     """Prompts for a directory path"""
-
     options = (
-        QtWidgets.QFileDialog.ShowDirsOnly | QtWidgets.QFileDialog.DontResolveSymlinks
+        QtWidgets.QFileDialog.Directory
+        | QtWidgets.QFileDialog.DontResolveSymlinks
+        | QtWidgets.QFileDialog.ReadOnly
+        | QtWidgets.QFileDialog.ShowDirsOnly
     )
     return compat.getexistingdirectory(
         parent=active_window(), caption=caption, basedir=path, options=options
@@ -939,10 +941,10 @@ class Task(QtCore.QRunnable):
 
     """
 
-    def __init__(self, parent):
+    def __init__(self):
         QtCore.QRunnable.__init__(self)
 
-        self.channel = Channel(parent)
+        self.channel = Channel()
         self.result = None
         self.setAutoDelete(False)
 
@@ -962,8 +964,8 @@ class Task(QtCore.QRunnable):
 class SimpleTask(Task):
     """Run a simple callable as a task"""
 
-    def __init__(self, parent, fn, *args, **kwargs):
-        Task.__init__(self, parent)
+    def __init__(self, fn, *args, **kwargs):
+        Task.__init__(self)
 
         self.fn = fn
         self.args = args
