@@ -3,7 +3,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import sys
 import re
 from argparse import ArgumentParser
-from functools import partial
+from functools import partial, reduce
 
 from cola import app  # prints a message if Qt cannot be found
 from qtpy import QtCore
@@ -17,7 +17,6 @@ from cola import core
 from cola import difftool
 from cola import hotkeys
 from cola import icons
-from cola import observable
 from cola import qtutils
 from cola import utils
 from cola.i18n import N_
@@ -154,10 +153,9 @@ class Editor(QtWidgets.QWidget):
         self.comment_char = comment_char = prefs.comment_char(context)
         self.cancel_action = core.getenv('GIT_COLA_SEQ_EDITOR_CANCEL_ACTION', 'abort')
 
-        self.notifier = notifier = observable.Observable()
-        self.diff = diff.DiffWidget(context, notifier, self)
-        self.tree = RebaseTreeWidget(context, notifier, comment_char, self)
-        self.filewidget = filelist.FileWidget(context, notifier, self)
+        self.diff = diff.DiffWidget(context, self)
+        self.tree = RebaseTreeWidget(context, comment_char, self)
+        self.filewidget = filelist.FileWidget(context, self)
         self.setFocusProxy(self.tree)
 
         self.rebase_button = qtutils.create_button(
@@ -205,9 +203,13 @@ class Editor(QtWidgets.QWidget):
             self, N_('Rebase'), self.rebase, hotkeys.CTRL_RETURN, hotkeys.CTRL_ENTER
         )
 
-        notifier.add_observer(diff.COMMITS_SELECTED, self.commits_selected)
+        self.tree.commits_selected.connect(self.commits_selected)
+        self.tree.commits_selected.connect(self.filewidget.commits_selected)
+        self.tree.commits_selected.connect(self.diff.commits_selected)
         self.tree.external_diff.connect(self.external_diff)
 
+        self.filewidget.files_selected.connect(self.diff.files_selected)
+
         qtutils.connect_button(self.rebase_button, self.rebase)
         qtutils.connect_button(self.extdiff_button, self.external_diff)
         qtutils.connect_button(self.help_button, partial(show_help, context))
@@ -217,7 +219,7 @@ class Editor(QtWidgets.QWidget):
         insns = core.read(self.filename)
         self.parse_sequencer_instructions(insns)
 
-    # notifier callbacks
+    # signal callbacks
     def commits_selected(self, commits):
         self.extdiff_button.setEnabled(bool(commits))
 
@@ -298,13 +300,13 @@ class Editor(QtWidgets.QWidget):
 
 # pylint: disable=too-many-ancestors
 class RebaseTreeWidget(standard.DraggableTreeWidget):
+    commits_selected = Signal(object)
     external_diff = Signal()
     move_rows = Signal(object, object)
 
-    def __init__(self, context, notifier, comment_char, parent=None):
+    def __init__(self, context, comment_char, parent):
         super(RebaseTreeWidget, self).__init__(parent=parent)
         self.context = context
-        self.notifier = notifier
         self.comment_char = comment_char
         # header
         self.setHeaderLabels(
@@ -318,6 +320,7 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
         )
         self.header().setStretchLastSection(True)
         self.setColumnCount(5)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
 
         # actions
         self.copy_oid_action = qtutils.add_action(
@@ -441,13 +444,13 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
             commits.append(c)
         if commits:
             commits = commits[-1:]
-        self.notifier.notify_observers(diff.COMMITS_SELECTED, commits)
+        self.commits_selected.emit(commits)
 
     def toggle_enabled(self):
-        item = self.selected_item()
-        if item is None:
-            return
-        item.toggle_enabled()
+        items = self.selected_items()
+        logic_or = reduce(lambda res, item: res or item.is_enabled(), items, False)
+        for item in items:
+            item.set_enabled(not logic_or)
 
     def select_first(self):
         items = self.items()
@@ -458,40 +461,50 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
             self.setCurrentIndex(idx)
 
     def shift_down(self):
-        item = self.selected_item()
-        if item is None:
+        sel_items = self.selected_items()
+        all_items = self.items()
+        sel_idx = sorted([all_items.index(item) for item in sel_items])
+        if not sel_idx:
             return
-        items = self.items()
-        idx = items.index(item)
-        if idx < len(items) - 1:
-            self.move_rows.emit([idx], idx + 1)
+        idx = sel_idx[0] + 1
+        if not (
+            idx > len(all_items) - len(sel_items)
+            or all_items[sel_idx[-1]] is all_items[-1]
+        ):
+            self.move_rows.emit(sel_idx, idx)
 
     def shift_up(self):
-        item = self.selected_item()
-        if item is None:
+        sel_items = self.selected_items()
+        all_items = self.items()
+        sel_idx = sorted([all_items.index(item) for item in sel_items])
+        if not sel_idx:
             return
-        items = self.items()
-        idx = items.index(item)
-        if idx > 0:
-            self.move_rows.emit([idx], idx - 1)
+        idx = sel_idx[0] - 1
+        if idx >= 0:
+            self.move_rows.emit(sel_idx, idx)
 
     def move(self, src_idxs, dst_idx):
-        new_items = []
-        items = self.items()
+        moved_items = list()
+        src_base = sorted(src_idxs)[0]
         for idx in reversed(sorted(src_idxs)):
-            item = items[idx].copy()
-            self.invisibleRootItem().takeChild(idx)
-            new_items.insert(0, item)
+            item = self.invisibleRootItem().takeChild(idx)
+            moved_items.insert(0, [dst_idx + (idx - src_base), item])
 
-        if new_items:
-            self.invisibleRootItem().insertChildren(dst_idx, new_items)
-            self.setCurrentItem(new_items[0])
+        for item in moved_items:
+            self.invisibleRootItem().insertChild(item[0], item[1])
+            self.setCurrentItem(item[1])
+
+        if moved_items:
+            moved_items = [item[1] for item in moved_items]
             # If we've moved to the top then we need to re-decorate all items.
             # Otherwise, we can decorate just the new items.
             if dst_idx == 0:
                 self.decorate(self.items())
             else:
-                self.decorate(new_items)
+                self.decorate(moved_items)
+
+            for item in moved_items:
+                item.setSelected(True)
         self.validate()
 
     # Qt events
@@ -501,6 +514,7 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
         self.validate()
 
     def contextMenuEvent(self, event):
+        items = self.selected_items()
         menu = qtutils.create_menu(N_('Actions'), self)
         menu.addAction(self.action_pick)
         menu.addAction(self.action_reword)
@@ -511,7 +525,11 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
         menu.addAction(self.toggle_enabled_action)
         menu.addSeparator()
         menu.addAction(self.copy_oid_action)
+        if len(items) > 1:
+            self.copy_oid_action.setDisabled(True)
         menu.addAction(self.external_diff_action)
+        if len(items) > 1:
+            self.external_diff_action.setDisabled(True)
         menu.exec_(self.mapToGlobal(event.pos()))
 
 
