@@ -4,7 +4,6 @@ import os
 from functools import partial
 
 from qtpy.QtCore import Qt
-from qtpy.QtCore import Signal
 from qtpy import QtCore
 from qtpy import QtWidgets
 
@@ -108,12 +107,6 @@ class StatusWidget(QtWidgets.QFrame):
 
 # pylint: disable=too-many-ancestors
 class StatusTreeWidget(QtWidgets.QTreeWidget):
-    # Signals
-    about_to_update = Signal()
-    set_previous_contents = Signal(list, list, list, list)
-    updated = Signal()
-    diff_text_changed = Signal()
-
     # Read-only access to the mode state
     mode = property(lambda self: self.m.mode)
 
@@ -191,14 +184,15 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             context, self, *hotkeys.ACCEPT
         )
 
-        if not utils.is_win32():
-            self.default_app_action = common.default_app_action(
-                context, self, self.selected_group
-            )
+        self.default_app_action = common.default_app_action(
+            context, self, self.selected_group
+        )
 
-            self.parent_dir_action = common.parent_dir_action(
-                context, self, self.selected_group
-            )
+        self.parent_dir_action = common.parent_dir_action(
+            context, self, self.selected_group
+        )
+
+        self.worktree_dir_action = common.worktree_dir_action(context, self)
 
         self.terminal_action = common.terminal_action(
             context, self, self.selected_group
@@ -291,27 +285,16 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         )
         self.delete_untracked_files_action.setIcon(icons.discard())
 
-        self.about_to_update.connect(self._about_to_update, type=Qt.QueuedConnection)
-        self.set_previous_contents.connect(
-            self._set_previous_contents, type=Qt.QueuedConnection)
-        self.updated.connect(self.refresh, type=Qt.QueuedConnection)
-        self.diff_text_changed.connect(
-            self._make_current_item_visible, type=Qt.QueuedConnection
-        )
-
         # The model is stored as self.m because self.model() is a
         # QTreeWidgetItem method that returns a QAbstractItemModel.
         self.m = context.model
-        # Forward the previous_contents notification through self.set_previous_contents.
-        self.m.add_observer(
-            self.m.message_previous_contents, self.set_previous_contents.emit
+        self.m.previous_contents.connect(
+            self._set_previous_contents, type=Qt.QueuedConnection
         )
-        # Forward the about_to_update notification through self.about_to_update.
-        self.m.add_observer(self.m.message_about_to_update, self.about_to_update.emit)
-        # Forward the updated notification through self.updated.
-        self.m.add_observer(self.m.message_updated, self.updated.emit)
-        self.m.add_observer(
-            self.m.message_diff_text_changed, self.diff_text_changed.emit
+        self.m.about_to_update.connect(self._about_to_update, type=Qt.QueuedConnection)
+        self.m.updated.connect(self.refresh, type=Qt.QueuedConnection)
+        self.m.diff_text_changed.connect(
+            self._make_current_item_visible, type=Qt.QueuedConnection
         )
         # pylint: disable=no-member
         self.itemSelectionChanged.connect(self.show_selection)
@@ -356,9 +339,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         # The current/new set of categorized files.
         new_c = self.contents()
 
-        select_staged = partial(
-            _select_item, self, new_c.staged, self._staged_item
-        )
+        select_staged = partial(_select_item, self, new_c.staged, self._staged_item)
         select_unmerged = partial(
             _select_item, self, new_c.unmerged, self._unmerged_item
         )
@@ -467,9 +448,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         """Stage or unstage files according to the selection"""
         context = self.context
         selected_indexes = self.selected_indexes()
-        is_header = any(
-            category == HEADER_IDX for (category, idx) in selected_indexes
-        )
+        is_header = any(category == HEADER_IDX for (category, idx) in selected_indexes)
         if is_header:
             is_staged = any(
                 idx == STAGED_IDX and category == HEADER_IDX
@@ -628,9 +607,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
     def _set_untracked(self, items):
         """Adds items to the 'Untracked' subtree."""
         with qtutils.BlockSignals(self):
-            self._set_subtree(
-                items, UNTRACKED_IDX, N_('Untracked'), untracked=True
-            )
+            self._set_subtree(items, UNTRACKED_IDX, N_('Untracked'), untracked=True)
 
     def _set_subtree(
         self, items, idx, parent_title, staged=False, untracked=False, deleted_set=None
@@ -698,16 +675,18 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         else:
             self._create_unstaged_context_menu(menu, s)
 
-        if not utils.is_win32():
-            if not menu.isEmpty():
-                menu.addSeparator()
-            if not self.selection_model.is_empty():
-                menu.addAction(self.default_app_action)
-                menu.addAction(self.parent_dir_action)
+        if not menu.isEmpty():
+            menu.addSeparator()
+
+        if not self.selection_model.is_empty():
+            menu.addAction(self.default_app_action)
+            menu.addAction(self.parent_dir_action)
 
         if self.terminal_action is not None:
             menu.addAction(self.terminal_action)
 
+        menu.addAction(self.worktree_dir_action)
+
         self._add_copy_actions(menu)
 
         return menu
